# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(450, 323)
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName(u"actionOpen")
        self.actionSave = QAction(MainWindow)
        self.actionSave.setObjectName(u"actionSave")
        self.actionValidate = QAction(MainWindow)
        self.actionValidate.setObjectName(u"actionValidate")
        self.actionLogs = QAction(MainWindow)
        self.actionLogs.setObjectName(u"actionLogs")
        self.action_software10e_help = QAction(MainWindow)
        self.action_software10e_help.setObjectName(u"action_software10e_help")
        self.action_fcp_help = QAction(MainWindow)
        self.action_fcp_help.setObjectName(u"action_fcp_help")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.horizontalLayout = QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 9)
        self.tabWidget = QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.verticalLayout_3 = QVBoxLayout(self.tab)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalSpacer_2 = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer_2)

        self.scrollArea = QScrollArea(self.tab)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setMaximumSize(QSize(400, 16777215))
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaContents = QWidget()
        self.scrollAreaContents.setObjectName(u"scrollAreaContents")
        self.scrollAreaContents.setGeometry(QRect(0, 0, 368, 185))
        self.verticalLayout = QVBoxLayout(self.scrollAreaContents)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.scrollArea.setWidget(self.scrollAreaContents)

        self.horizontalLayout_3.addWidget(self.scrollArea)

        self.horizontalSpacer = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer)

        self.horizontalLayout_3.setStretch(1, 10)

        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.addButton = QPushButton(self.tab)
        self.addButton.setObjectName(u"addButton")
        self.addButton.setMaximumSize(QSize(400, 16777215))

        self.horizontalLayout_2.addWidget(self.addButton)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)


        self.verticalLayout_3.addLayout(self.verticalLayout_2)

        self.tabWidget.addTab(self.tab, "")
        self.Logs = QWidget()
        self.Logs.setObjectName(u"Logs")
        self.verticalLayout_4 = QVBoxLayout(self.Logs)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.logDetailsLayout = QVBoxLayout()
        self.logDetailsLayout.setObjectName(u"logDetailsLayout")

        self.verticalLayout_4.addLayout(self.logDetailsLayout)

        self.tabWidget.addTab(self.Logs, "")
        self.tab_3 = QWidget()
        self.tab_3.setObjectName(u"tab_3")
        self.verticalLayout_5 = QVBoxLayout(self.tab_3)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.enumDetailsLayout = QVBoxLayout()
        self.enumDetailsLayout.setObjectName(u"enumDetailsLayout")

        self.verticalLayout_5.addLayout(self.enumDetailsLayout)

        self.tabWidget.addTab(self.tab_3, "")

        self.horizontalLayout.addWidget(self.tabWidget)

        self.deviceDetailsLayout = QVBoxLayout()
        self.deviceDetailsLayout.setObjectName(u"deviceDetailsLayout")

        self.horizontalLayout.addLayout(self.deviceDetailsLayout)

        self.messageDetailsLayout = QVBoxLayout()
        self.messageDetailsLayout.setObjectName(u"messageDetailsLayout")

        self.horizontalLayout.addLayout(self.messageDetailsLayout)

        self.signalDetailsLayout = QVBoxLayout()
        self.signalDetailsLayout.setObjectName(u"signalDetailsLayout")

        self.horizontalLayout.addLayout(self.signalDetailsLayout)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 450, 18))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        self.menuHelp = QMenu(self.menubar)
        self.menuHelp.setObjectName(u"menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionValidate)
        self.menuHelp.addAction(self.action_software10e_help)
        self.menuHelp.addAction(self.action_fcp_help)

        self.retranslateUi(MainWindow)

        self.tabWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", u"Open", None))
        self.actionSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.actionValidate.setText(QCoreApplication.translate("MainWindow", u"Validate", None))
        self.actionLogs.setText(QCoreApplication.translate("MainWindow", u"Logs", None))
        self.action_software10e_help.setText(QCoreApplication.translate("MainWindow", u"Software10e FCP", None))
        self.action_fcp_help.setText(QCoreApplication.translate("MainWindow", u"FCP Help", None))
        self.addButton.setText(QCoreApplication.translate("MainWindow", u"Add", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), QCoreApplication.translate("MainWindow", u"Devices", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Logs), QCoreApplication.translate("MainWindow", u"Logs", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), QCoreApplication.translate("MainWindow", u"Enums", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menuHelp.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'messagedetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_MessageDetails(object):
    def setupUi(self, MessageDetails):
        if not MessageDetails.objectName():
            MessageDetails.setObjectName(u"MessageDetails")
        MessageDetails.resize(342, 308)
        self.verticalLayout = QVBoxLayout(MessageDetails)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.groupBox = QGroupBox(MessageDetails)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout_3 = QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.nameLabel = QLabel(self.groupBox)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(self.groupBox)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout.addWidget(self.nameEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.idLabel = QLabel(self.groupBox)
        self.idLabel.setObjectName(u"idLabel")

        self.horizontalLayout_2.addWidget(self.idLabel)

        self.idEdit = QLineEdit(self.groupBox)
        self.idEdit.setObjectName(u"idEdit")

        self.horizontalLayout_2.addWidget(self.idEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.dlcLabel = QLabel(self.groupBox)
        self.dlcLabel.setObjectName(u"dlcLabel")

        self.horizontalLayout_3.addWidget(self.dlcLabel)

        self.dlcEdit = QLineEdit(self.groupBox)
        self.dlcEdit.setObjectName(u"dlcEdit")

        self.horizontalLayout_3.addWidget(self.dlcEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.frequencyLabel = QLabel(self.groupBox)
        self.frequencyLabel.setObjectName(u"frequencyLabel")

        self.horizontalLayout_4.addWidget(self.frequencyLabel)

        self.frequencyEdit = QLineEdit(self.groupBox)
        self.frequencyEdit.setObjectName(u"frequencyEdit")

        self.horizontalLayout_4.addWidget(self.frequencyEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.descriptionLabel = QLabel(self.groupBox)
        self.descriptionLabel.setObjectName(u"descriptionLabel")

        self.horizontalLayout_6.addWidget(self.descriptionLabel)

        self.descriptionEdit = QLineEdit(self.groupBox)
        self.descriptionEdit.setObjectName(u"descriptionEdit")

        self.horizontalLayout_6.addWidget(self.descriptionEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout_6)

        self.scrollArea = QScrollArea(self.groupBox)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.signalContents = QWidget()
        self.signalContents.setObjectName(u"signalContents")
        self.signalContents.setGeometry(QRect(0, 0, 298, 60))
        self.verticalLayout_2 = QVBoxLayout(self.signalContents)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.scrollArea.setWidget(self.signalContents)

        self.verticalLayout_3.addWidget(self.scrollArea)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.addButton = QPushButton(self.groupBox)
        self.addButton.setObjectName(u"addButton")

        self.horizontalLayout_5.addWidget(self.addButton)

        self.deleteMessageButton = QPushButton(self.groupBox)
        self.deleteMessageButton.setObjectName(u"deleteMessageButton")

        self.horizontalLayout_5.addWidget(self.deleteMessageButton)


        self.verticalLayout_3.addLayout(self.horizontalLayout_5)


        self.verticalLayout.addWidget(self.groupBox)


        self.retranslateUi(MessageDetails)

        QMetaObject.connectSlotsByName(MessageDetails)
    # setupUi

    def retranslateUi(self, MessageDetails):
        MessageDetails.setWindowTitle(QCoreApplication.translate("MessageDetails", u"MessageDetails", None))
        self.groupBox.setTitle(QCoreApplication.translate("MessageDetails", u"Message", None))
        self.nameLabel.setText(QCoreApplication.translate("MessageDetails", u"name", None))
        self.idLabel.setText(QCoreApplication.translate("MessageDetails", u"id", None))
        self.dlcLabel.setText(QCoreApplication.translate("MessageDetails", u"dlc", None))
        self.frequencyLabel.setText(QCoreApplication.translate("MessageDetails", u"frequency", None))
        self.descriptionLabel.setText(QCoreApplication.translate("MessageDetails", u"description", None))
        self.addButton.setText(QCoreApplication.translate("MessageDetails", u"Add", None))
        self.deleteMessageButton.setText(QCoreApplication.translate("MessageDetails", u"Delete", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'messagewidget.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_MessageWidget(object):
    def setupUi(self, MessageWidget):
        if not MessageWidget.objectName():
            MessageWidget.setObjectName(u"MessageWidget")
        MessageWidget.resize(388, 92)
        self.horizontalLayout = QHBoxLayout(MessageWidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.groupBox = QGroupBox(MessageWidget)
        self.groupBox.setObjectName(u"groupBox")
        self.horizontalLayout_4 = QHBoxLayout(self.groupBox)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.name = QLabel(self.groupBox)
        self.name.setObjectName(u"name")

        self.horizontalLayout_2.addWidget(self.name)

        self.id = QLabel(self.groupBox)
        self.id.setObjectName(u"id")

        self.horizontalLayout_2.addWidget(self.id)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.dlc = QLabel(self.groupBox)
        self.dlc.setObjectName(u"dlc")

        self.horizontalLayout_3.addWidget(self.dlc)

        self.frequency = QLabel(self.groupBox)
        self.frequency.setObjectName(u"frequency")

        self.horizontalLayout_3.addWidget(self.frequency)


        self.verticalLayout.addLayout(self.horizontalLayout_3)


        self.horizontalLayout_4.addLayout(self.verticalLayout)

        self.messageDetailsButton = QPushButton(self.groupBox)
        self.messageDetailsButton.setObjectName(u"messageDetailsButton")
        self.messageDetailsButton.setMaximumSize(QSize(60, 25))
        self.messageDetailsButton.setCheckable(True)

        self.horizontalLayout_4.addWidget(self.messageDetailsButton)


        self.horizontalLayout.addWidget(self.groupBox)


        self.retranslateUi(MessageWidget)

        QMetaObject.connectSlotsByName(MessageWidget)
    # setupUi

    def retranslateUi(self, MessageWidget):
        MessageWidget.setWindowTitle(QCoreApplication.translate("MessageWidget", u"MessageWidget", None))
        self.groupBox.setTitle("")
        self.name.setText("")
        self.id.setText("")
        self.dlc.setText("")
        self.frequency.setText("")
        self.messageDetailsButton.setText(QCoreApplication.translate("MessageWidget", u"...", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'cfgdetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_CfgDetails(object):
    def setupUi(self, CfgDetails):
        if not CfgDetails.objectName():
            CfgDetails.setObjectName(u"CfgDetails")
        CfgDetails.resize(400, 300)
        CfgDetails.setMinimumSize(QSize(200, 0))
        self.verticalLayout_2 = QVBoxLayout(CfgDetails)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.groupBox = QGroupBox(CfgDetails)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout = QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.nameLabel = QLabel(self.groupBox)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout_3.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(self.groupBox)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout_3.addWidget(self.nameEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.idLabel_2 = QLabel(self.groupBox)
        self.idLabel_2.setObjectName(u"idLabel_2")

        self.horizontalLayout_2.addWidget(self.idLabel_2)

        self.idEdit = QLineEdit(self.groupBox)
        self.idEdit.setObjectName(u"idEdit")

        self.horizontalLayout_2.addWidget(self.idEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.commentLabel = QLabel(self.groupBox)
        self.commentLabel.setObjectName(u"commentLabel")

        self.horizontalLayout.addWidget(self.commentLabel)

        self.commentEdit = QLineEdit(self.groupBox)
        self.commentEdit.setObjectName(u"commentEdit")

        self.horizontalLayout.addWidget(self.commentEdit)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.deleteCfgButton = QPushButton(self.groupBox)
        self.deleteCfgButton.setObjectName(u"deleteCfgButton")

        self.verticalLayout.addWidget(self.deleteCfgButton)


        self.verticalLayout_2.addWidget(self.groupBox)


        self.retranslateUi(CfgDetails)

        QMetaObject.connectSlotsByName(CfgDetails)
    # setupUi

    def retranslateUi(self, CfgDetails):
        CfgDetails.setWindowTitle(QCoreApplication.translate("CfgDetails", u"CfgDetails", None))
        self.groupBox.setTitle(QCoreApplication.translate("CfgDetails", u"Config", None))
        self.nameLabel.setText(QCoreApplication.translate("CfgDetails", u"name", None))
        self.idLabel_2.setText(QCoreApplication.translate("CfgDetails", u"id", None))
        self.commentLabel.setText(QCoreApplication.translate("CfgDetails", u"comment", None))
        self.deleteCfgButton.setText(QCoreApplication.translate("CfgDetails", u"Delete", None))
    # retranslateUi




import json
from fcp.spec import *
from fcp.can import CANMessage

from math import *


def bitmask(n):
    return 2 ** n - 1

def test_bitmask():
    assert 0xF == bitmask(4)


def encode_signal(signal, value):
    value = (value - signal.offset) / signal.scale
    return (int(value) & bitmask(signal.length)) << signal.start


def conv_endianess(value: int, signal: Signal):
    length = signal.get_length()/8
    log2_length = log2(length)
    
    if length == 1:
        return value

    if log2_length != int(log2_length):
        return value

    if signal.get_byte_order() != "big_endian":
        return value

    length = int(length)
    def swap64(x):
        return int.from_bytes(value.to_bytes(8, byteorder='little'), byteorder='big', signed=False)
    def swap32(x):
        return int.from_bytes(value.to_bytes(4, byteorder='little'), byteorder='big', signed=False)
    def swap16(x):
        return int.from_bytes(value.to_bytes(2, byteorder='little'), byteorder='big', signed=False)

    if length == 8:
        return swap64(value)
    if length == 4:
        return swap32(value) 
    if length == 2:
        return swap16(value)


def decode_signal(signal, value):
    value = (value >> signal.start) & bitmask(signal.length)
    value = conv_endianess(value, signal)
    if signal.type == 'signed':
        value = int(value)
        if (value >> (signal.length - 1)) == 1:
            value = - ((value ^ bitmask(signal.length)) + 1)
    return (value * signal.scale) + signal.offset


class Fcp:
    def __init__(self, spec):
        self.messages = {}
        self.spec = spec

        for device in spec.devices.values():
            for msg in device.msgs.values():
                self.messages[msg.name] = msg

        for msg in spec.common.msgs.values():
            self.messages[msg.name] = msg

    def encode_msg(self, sid: int, msg_name: str, signals: Dict[str, float]) -> CANMessage:
        msg = self.messages.get(msg_name)

        data = 0

        sigs = msg.signals

        for name, value in signals.items():
            signal = sigs[name]
            data |= encode_signal(signal, value)

        sid = make_sid(sid, msg.id)
        return CANMessage(sid, msg.dlc, 1, data64 = data)

    def find_msg(self, msg):
        dev_id, msg_id = decompose_id(msg.sid)

        for dev in self.spec.devices.values():
            if dev.id == dev_id:
                for msg in dev.msgs.values():
                    if msg.id == msg_id:
                        return msg

        for msg in self.spec.common.msgs.values():
            if msg.id == msg_id:
                return msg
    
    def find_device(self, id):
        for dev in self.spec.devices.values:
            if isinstance(id, int) and dev.id == id:
                return dev
            if isinstance(id, str) and dev.name == id:
                return dev

        return None

    def find_config(self, dev, id):
        for name, cfg in dev.cfgs.items():
            if isinstance(id, str) and name == id:
                return cfg

        return None


    def decode_msg(self, msg: CANMessage):
        fcp_msg = self.find_msg(msg)
        if fcp_msg == None:
            return "", {}

        signals = {}
        data = 0
        for i, d in enumerate(msg.get_data16()):
            data += d << 16 * i

        for name, signal in fcp_msg.signals.items():
            signals[name] = decode_signal(signal, data)

        return fcp_msg.name, signals

    def decode_log(self, signal):
        for name, log in self.spec.logs.items():
            if log.id == signal["id"]:
                return log.string

        return ""

    def decode_get(self, signal):
        dev = self.find_device(signal['dst'])
        if dev == None:
            return None

        for name, config in dev.cfgs.items():
            if config.id == signal['id']:
                return {config.name : signal['data']}

    
    def encode_get(self, sid: int, dst: int, config: int):
        return encode_msg(sid, "req_get", {"dst": dst, "id": config})

    def decode_set(self, signal):
        dev = self.find_device(signal['dst'])
        if dev == None:
            return None

        for name, config in dev.cfgs.items():
            if config.id == signal['id']:
                return {"device": device.name, config.name : signal['data']}

    
    def encode_set(self, sid: int, dst: int, config: int, value: int):
        return encode_msg(sid, "req_set", {"dst": dst, "id": config, "value": value})

#class FCPCom():
#    def __init__(self, fcp, com, sid):
#        self.fcp = fcp
#        self.com = com
#        self.sid = sid
#
#    def get(self, device, config):
#        dev = self.fcp.find_device(device)
#        if dev == None:
#            return None
#
#        cfg = self.fcp.find_config(dev, config)
#        if cfg == None:
#            return None
#
#        msg = self.fcp.encode_get(self.sid, dev.id, cfg.id)
#
#        com.send(msg)







# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'signaldetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_SignalDetails(object):
    def setupUi(self, SignalDetails):
        if not SignalDetails.objectName():
            SignalDetails.setObjectName(u"SignalDetails")
        SignalDetails.resize(473, 580)
        self.verticalLayout_2 = QVBoxLayout(SignalDetails)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.groupBox = QGroupBox(SignalDetails)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout = QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_14 = QHBoxLayout()
        self.horizontalLayout_14.setObjectName(u"horizontalLayout_14")
        self.nameLabel = QLabel(self.groupBox)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout_14.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(self.groupBox)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout_14.addWidget(self.nameEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_14)

        self.horizontalLayout_15 = QHBoxLayout()
        self.horizontalLayout_15.setObjectName(u"horizontalLayout_15")
        self.aliasLabel = QLabel(self.groupBox)
        self.aliasLabel.setObjectName(u"aliasLabel")

        self.horizontalLayout_15.addWidget(self.aliasLabel)

        self.aliasEdit = QLineEdit(self.groupBox)
        self.aliasEdit.setObjectName(u"aliasEdit")

        self.horizontalLayout_15.addWidget(self.aliasEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_15)

        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setObjectName(u"horizontalLayout_13")
        self.commentLabel = QLabel(self.groupBox)
        self.commentLabel.setObjectName(u"commentLabel")

        self.horizontalLayout_13.addWidget(self.commentLabel)

        self.commentEdit = QLineEdit(self.groupBox)
        self.commentEdit.setObjectName(u"commentEdit")

        self.horizontalLayout_13.addWidget(self.commentEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_13)

        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setObjectName(u"horizontalLayout_12")
        self.startLabel = QLabel(self.groupBox)
        self.startLabel.setObjectName(u"startLabel")

        self.horizontalLayout_12.addWidget(self.startLabel)

        self.startEdit = QLineEdit(self.groupBox)
        self.startEdit.setObjectName(u"startEdit")

        self.horizontalLayout_12.addWidget(self.startEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_12)

        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setObjectName(u"horizontalLayout_11")
        self.lengthLabel = QLabel(self.groupBox)
        self.lengthLabel.setObjectName(u"lengthLabel")

        self.horizontalLayout_11.addWidget(self.lengthLabel)

        self.lengthEdit = QLineEdit(self.groupBox)
        self.lengthEdit.setObjectName(u"lengthEdit")

        self.horizontalLayout_11.addWidget(self.lengthEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_11)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.typeLabel = QLabel(self.groupBox)
        self.typeLabel.setObjectName(u"typeLabel")

        self.horizontalLayout_9.addWidget(self.typeLabel)

        self.typeEdit = QLineEdit(self.groupBox)
        self.typeEdit.setObjectName(u"typeEdit")

        self.horizontalLayout_9.addWidget(self.typeEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_9)

        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.unitLabel = QLabel(self.groupBox)
        self.unitLabel.setObjectName(u"unitLabel")

        self.horizontalLayout_8.addWidget(self.unitLabel)

        self.unitEdit = QLineEdit(self.groupBox)
        self.unitEdit.setObjectName(u"unitEdit")

        self.horizontalLayout_8.addWidget(self.unitEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_8)

        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.minValueLabel = QLabel(self.groupBox)
        self.minValueLabel.setObjectName(u"minValueLabel")

        self.horizontalLayout_7.addWidget(self.minValueLabel)

        self.minValueEdit = QLineEdit(self.groupBox)
        self.minValueEdit.setObjectName(u"minValueEdit")

        self.horizontalLayout_7.addWidget(self.minValueEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_7)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.maxValueLabel = QLabel(self.groupBox)
        self.maxValueLabel.setObjectName(u"maxValueLabel")

        self.horizontalLayout_6.addWidget(self.maxValueLabel)

        self.maxValueEdit = QLineEdit(self.groupBox)
        self.maxValueEdit.setObjectName(u"maxValueEdit")

        self.horizontalLayout_6.addWidget(self.maxValueEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.muxLabel = QLabel(self.groupBox)
        self.muxLabel.setObjectName(u"muxLabel")

        self.horizontalLayout_5.addWidget(self.muxLabel)

        self.muxEdit = QLineEdit(self.groupBox)
        self.muxEdit.setObjectName(u"muxEdit")

        self.horizontalLayout_5.addWidget(self.muxEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.muxCountLabel = QLabel(self.groupBox)
        self.muxCountLabel.setObjectName(u"muxCountLabel")

        self.horizontalLayout_4.addWidget(self.muxCountLabel)

        self.muxCountEdit = QLineEdit(self.groupBox)
        self.muxCountEdit.setObjectName(u"muxCountEdit")

        self.horizontalLayout_4.addWidget(self.muxCountEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.byteOrderLabel = QLabel(self.groupBox)
        self.byteOrderLabel.setObjectName(u"byteOrderLabel")

        self.horizontalLayout_3.addWidget(self.byteOrderLabel)

        self.byteOrderEdit = QLineEdit(self.groupBox)
        self.byteOrderEdit.setObjectName(u"byteOrderEdit")

        self.horizontalLayout_3.addWidget(self.byteOrderEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.scaleLabel = QLabel(self.groupBox)
        self.scaleLabel.setObjectName(u"scaleLabel")

        self.horizontalLayout_2.addWidget(self.scaleLabel)

        self.scaleEdit = QLineEdit(self.groupBox)
        self.scaleEdit.setObjectName(u"scaleEdit")

        self.horizontalLayout_2.addWidget(self.scaleEdit)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.offsetLabel = QLabel(self.groupBox)
        self.offsetLabel.setObjectName(u"offsetLabel")

        self.horizontalLayout.addWidget(self.offsetLabel)

        self.offsetEdit = QLineEdit(self.groupBox)
        self.offsetEdit.setObjectName(u"offsetEdit")

        self.horizontalLayout.addWidget(self.offsetEdit)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.saveSignal = QPushButton(self.groupBox)
        self.saveSignal.setObjectName(u"saveSignal")

        self.horizontalLayout_10.addWidget(self.saveSignal)

        self.signalDeleteButton = QPushButton(self.groupBox)
        self.signalDeleteButton.setObjectName(u"signalDeleteButton")

        self.horizontalLayout_10.addWidget(self.signalDeleteButton)


        self.verticalLayout.addLayout(self.horizontalLayout_10)


        self.verticalLayout_2.addWidget(self.groupBox)


        self.retranslateUi(SignalDetails)

        QMetaObject.connectSlotsByName(SignalDetails)
    # setupUi

    def retranslateUi(self, SignalDetails):
        SignalDetails.setWindowTitle(QCoreApplication.translate("SignalDetails", u"SignalDetails", None))
        self.groupBox.setTitle(QCoreApplication.translate("SignalDetails", u"Signal", None))
        self.nameLabel.setText(QCoreApplication.translate("SignalDetails", u"name", None))
        self.aliasLabel.setText(QCoreApplication.translate("SignalDetails", u"alias", None))
        self.commentLabel.setText(QCoreApplication.translate("SignalDetails", u"comment", None))
        self.startLabel.setText(QCoreApplication.translate("SignalDetails", u"start", None))
        self.lengthLabel.setText(QCoreApplication.translate("SignalDetails", u"length", None))
        self.typeLabel.setText(QCoreApplication.translate("SignalDetails", u"type", None))
        self.unitLabel.setText(QCoreApplication.translate("SignalDetails", u"unit", None))
        self.minValueLabel.setText(QCoreApplication.translate("SignalDetails", u"min_value", None))
        self.maxValueLabel.setText(QCoreApplication.translate("SignalDetails", u"max_value", None))
        self.muxLabel.setText(QCoreApplication.translate("SignalDetails", u"mux", None))
        self.muxCountLabel.setText(QCoreApplication.translate("SignalDetails", u"mux_count", None))
        self.byteOrderLabel.setText(QCoreApplication.translate("SignalDetails", u"byte_order", None))
        self.scaleLabel.setText(QCoreApplication.translate("SignalDetails", u"scale", None))
        self.offsetLabel.setText(QCoreApplication.translate("SignalDetails", u"offset", None))
        self.saveSignal.setText(QCoreApplication.translate("SignalDetails", u"Save", None))
        self.signalDeleteButton.setText(QCoreApplication.translate("SignalDetails", u"Delete", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'devicedetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_DeviceDetails(object):
    def setupUi(self, DeviceDetails):
        if not DeviceDetails.objectName():
            DeviceDetails.setObjectName(u"DeviceDetails")
        DeviceDetails.resize(398, 382)
        self.verticalLayout_2 = QVBoxLayout(DeviceDetails)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.groupBox = QGroupBox(DeviceDetails)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout_5 = QVBoxLayout(self.groupBox)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.nameLabel = QLabel(self.groupBox)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(self.groupBox)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout.addWidget(self.nameEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.idLabel = QLabel(self.groupBox)
        self.idLabel.setObjectName(u"idLabel")

        self.horizontalLayout_2.addWidget(self.idLabel)

        self.idEdit = QLineEdit(self.groupBox)
        self.idEdit.setObjectName(u"idEdit")

        self.horizontalLayout_2.addWidget(self.idEdit)


        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.scrollArea = QScrollArea(self.groupBox)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.messageContents = QWidget()
        self.messageContents.setObjectName(u"messageContents")
        self.messageContents.setGeometry(QRect(0, 0, 346, 202))
        self.verticalLayout = QVBoxLayout(self.messageContents)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.scrollArea.setWidget(self.messageContents)

        self.verticalLayout_3.addWidget(self.scrollArea)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.addButton = QPushButton(self.groupBox)
        self.addButton.setObjectName(u"addButton")

        self.horizontalLayout_5.addWidget(self.addButton)

        self.deleteDeviceButton = QPushButton(self.groupBox)
        self.deleteDeviceButton.setObjectName(u"deleteDeviceButton")

        self.horizontalLayout_5.addWidget(self.deleteDeviceButton)

        self.cmdsButton = QPushButton(self.groupBox)
        self.cmdsButton.setObjectName(u"cmdsButton")
        self.cmdsButton.setCheckable(True)

        self.horizontalLayout_5.addWidget(self.cmdsButton)

        self.cfgsButton = QPushButton(self.groupBox)
        self.cfgsButton.setObjectName(u"cfgsButton")
        self.cfgsButton.setCheckable(True)

        self.horizontalLayout_5.addWidget(self.cfgsButton)


        self.verticalLayout_3.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")

        self.verticalLayout_3.addLayout(self.horizontalLayout_4)


        self.horizontalLayout_3.addLayout(self.verticalLayout_3)

        self.cfgDetails = QVBoxLayout()
        self.cfgDetails.setObjectName(u"cfgDetails")

        self.horizontalLayout_3.addLayout(self.cfgDetails)

        self.cmdDetails = QVBoxLayout()
        self.cmdDetails.setObjectName(u"cmdDetails")

        self.horizontalLayout_3.addLayout(self.cmdDetails)


        self.verticalLayout_5.addLayout(self.horizontalLayout_3)


        self.verticalLayout_2.addWidget(self.groupBox)


        self.retranslateUi(DeviceDetails)

        QMetaObject.connectSlotsByName(DeviceDetails)
    # setupUi

    def retranslateUi(self, DeviceDetails):
        DeviceDetails.setWindowTitle(QCoreApplication.translate("DeviceDetails", u"DeviceDetails", None))
        self.groupBox.setTitle(QCoreApplication.translate("DeviceDetails", u"Device", None))
        self.nameLabel.setText(QCoreApplication.translate("DeviceDetails", u"name", None))
        self.idLabel.setText(QCoreApplication.translate("DeviceDetails", u"id", None))
        self.addButton.setText(QCoreApplication.translate("DeviceDetails", u"Add", None))
        self.deleteDeviceButton.setText(QCoreApplication.translate("DeviceDetails", u"Delete", None))
        self.cmdsButton.setText(QCoreApplication.translate("DeviceDetails", u"Cmds", None))
        self.cfgsButton.setText(QCoreApplication.translate("DeviceDetails", u"Cfgs", None))
    # retranslateUi




#!/usr/bin/env python3

"""c-generator.

Usage:
  c-generator.py generate <json> <template> <skel> <output>
  c-generator.py (-h | --help)
  c-generator.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.
"""

import sys, os
import json

import logging
import shutil

from .spec import Spec
from .template import Tpl
from .generator import *
from .validator import validate


def check_output(output):
    if not os.path.isdir(output):
        try:
            os.makedirs(output)
        except Exception as e:
            return False, str(e)

    return True, ""


def write_files(output, files):
    def gen_output(d):
        return os.path.join(output, d)
    
    for name, content in files:
        if content == "":
            continue
        with open(gen_output(name), "w") as f:
            f.write(content)

def check_version(j, logger):
    version = j.get("version")

    if version == None:
        logger.error("No version found")
        exit()

    if version != "0.2":
        logger.error("Wrong file version expected 0.2")
        exit()

    logger.info("Correct file version ✅")


def c_gen(templates, output, json_file, skel, logger):
    # copy skel directory
    status, err = check_output(output)
    if not status:
        logger.error(err)
        exit()

    for file in os.listdir(skel):
        shutil.copy(os.path.join(skel, file), output, follow_symlinks=True)

    tpl = Tpl(logger, templates)
    status, err = tpl.check_tpl_dir()
    if not status:
        logger.error(err)

    logger.info("Template dir checked ✅")

    tpl.open_tpl_files()

    with open(json_file) as f:
        r = f.read()

    logger.info("JSON spec file successfully read ✅")

    j = json.loads(r)
    ans = validate(logger, j)
    if not ans:
        return
    logger.info("Validated JSON ✅")
    spec = Spec()
    spec.decompile(j)

    check_version(j, logger)

    logger.info("JSON loaded into Spec ✅")
    

    files = []

    can_ids = build_can_ids(spec, tpl)
    for device in spec.devices.values():
        files += build_devices(spec, device, tpl)

    common = build_common(spec, tpl)
    enums = build_enums(spec, tpl)

    files += can_ids + common + enums

    write_files(output, files)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'enumdetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_EnumDetails(object):
    def setupUi(self, EnumDetails):
        if not EnumDetails.objectName():
            EnumDetails.setObjectName(u"EnumDetails")
        EnumDetails.resize(418, 288)
        self.verticalLayout = QVBoxLayout(EnumDetails)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.nameLabel = QLabel(EnumDetails)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout_2.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(EnumDetails)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout_2.addWidget(self.nameEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.enumContents = QScrollArea(EnumDetails)
        self.enumContents.setObjectName(u"enumContents")
        self.enumContents.setWidgetResizable(True)
        self._EnumValueContents = QWidget()
        self._EnumValueContents.setObjectName(u"_EnumValueContents")
        self._EnumValueContents.setGeometry(QRect(0, 0, 396, 204))
        self.verticalLayout_4 = QVBoxLayout(self._EnumValueContents)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.EnumValueContents = QVBoxLayout()
        self.EnumValueContents.setObjectName(u"EnumValueContents")

        self.verticalLayout_4.addLayout(self.EnumValueContents)

        self.enumContents.setWidget(self._EnumValueContents)

        self.verticalLayout_2.addWidget(self.enumContents)


        self.verticalLayout.addLayout(self.verticalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.enumDeleteButton = QPushButton(EnumDetails)
        self.enumDeleteButton.setObjectName(u"enumDeleteButton")

        self.horizontalLayout.addWidget(self.enumDeleteButton)

        self.valueAddButton = QPushButton(EnumDetails)
        self.valueAddButton.setObjectName(u"valueAddButton")

        self.horizontalLayout.addWidget(self.valueAddButton)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.retranslateUi(EnumDetails)

        QMetaObject.connectSlotsByName(EnumDetails)
    # setupUi

    def retranslateUi(self, EnumDetails):
        EnumDetails.setWindowTitle(QCoreApplication.translate("EnumDetails", u"EnumValue", None))
        self.nameLabel.setText(QCoreApplication.translate("EnumDetails", u"name", None))
        self.enumDeleteButton.setText(QCoreApplication.translate("EnumDetails", u"Delete", None))
        self.valueAddButton.setText(QCoreApplication.translate("EnumDetails", u"Add Value", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'logdetails.ui'
##
## Created by: Qt User Interface Compiler version 5.15.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QDate, QDateTime, QMetaObject,
    QObject, QPoint, QRect, QSize, QTime, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter,
    QPixmap, QRadialGradient)
from PySide2.QtWidgets import *


class Ui_LogDetails(object):
    def setupUi(self, LogDetails):
        if not LogDetails.objectName():
            LogDetails.setObjectName(u"LogDetails")
        LogDetails.resize(400, 300)
        LogDetails.setMinimumSize(QSize(200, 0))
        self.verticalLayout = QVBoxLayout(LogDetails)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.groupBox = QGroupBox(LogDetails)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout_2 = QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.nameLabel = QLabel(self.groupBox)
        self.nameLabel.setObjectName(u"nameLabel")

        self.horizontalLayout_2.addWidget(self.nameLabel)

        self.nameEdit = QLineEdit(self.groupBox)
        self.nameEdit.setObjectName(u"nameEdit")

        self.horizontalLayout_2.addWidget(self.nameEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.idLabel = QLabel(self.groupBox)
        self.idLabel.setObjectName(u"idLabel")

        self.horizontalLayout_3.addWidget(self.idLabel)

        self.idEdit = QLineEdit(self.groupBox)
        self.idEdit.setObjectName(u"idEdit")

        self.horizontalLayout_3.addWidget(self.idEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.n_argsLabel = QLabel(self.groupBox)
        self.n_argsLabel.setObjectName(u"n_argsLabel")

        self.horizontalLayout_4.addWidget(self.n_argsLabel)

        self.n_argsEdit = QLineEdit(self.groupBox)
        self.n_argsEdit.setObjectName(u"n_argsEdit")

        self.horizontalLayout_4.addWidget(self.n_argsEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.commentLabel = QLabel(self.groupBox)
        self.commentLabel.setObjectName(u"commentLabel")

        self.horizontalLayout_5.addWidget(self.commentLabel)

        self.commentEdit = QLineEdit(self.groupBox)
        self.commentEdit.setObjectName(u"commentEdit")

        self.horizontalLayout_5.addWidget(self.commentEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout_5)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.stringLabel = QLabel(self.groupBox)
        self.stringLabel.setObjectName(u"stringLabel")

        self.horizontalLayout.addWidget(self.stringLabel)

        self.stringEdit = QLineEdit(self.groupBox)
        self.stringEdit.setObjectName(u"stringEdit")

        self.horizontalLayout.addWidget(self.stringEdit)


        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.logDeleteButton = QPushButton(self.groupBox)
        self.logDeleteButton.setObjectName(u"logDeleteButton")

        self.verticalLayout_2.addWidget(self.logDeleteButton)


        self.verticalLayout.addWidget(self.groupBox)


        self.retranslateUi(LogDetails)

        QMetaObject.connectSlotsByName(LogDetails)
    # setupUi

    def retranslateUi(self, LogDetails):
        LogDetails.setWindowTitle(QCoreApplication.translate("LogDetails", u"LogDetails", None))
        self.groupBox.setTitle(QCoreApplication.translate("LogDetails", u"Log", None))
        self.nameLabel.setText(QCoreApplication.translate("LogDetails", u"name", None))
        self.idLabel.setText(QCoreApplication.translate("LogDetails", u"id", None))
        self.n_argsLabel.setText(QCoreApplication.translate("LogDetails", u"n_args", None))
        self.commentLabel.setText(QCoreApplication.translate("LogDetails", u"comment", None))
        self.stringLabel.setText(QCoreApplication.translate("LogDetails", u"string", None))
        self.logDeleteButton.setText(QCoreApplication.translate("LogDetails", u"Delete", None))
    # retranslateUi




import os
from jinja2 import Template

class Tpl:
    def __init__(self, logger, template_dir):
        self.files = {}

        self.expected_files = [
            "h.jinja",
            "c.jinja",
            "can_ids_h.jinja",
            "can_ids_c.jinja",
            "common_c.jinja",
            "common_h.jinja",
            "enums_h.jinja"
        ]

        self.template_dir = template_dir
        self.tpl = {}

        self.logger = logger
        return

    def check_tpl_dir(self):
        files = os.listdir(self.template_dir)

        for k in self.expected_files:
            if k not in files:
                return False, f"{k} missing"

        return True, ""

    def _get_tpl_dir_files(self):
        files = os.listdir(self.template_dir)
        expand_files = {}

        for k in self.expected_files:
            expand_files[k] = os.path.join(self.template_dir, k)

        return expand_files
    
    def render(self, name, *args, **kwargs):
        template = self.tpl.get(name)
        if template == None:
            self.logger.error(f"Error: Template not found, {name}")
            return ""

        return template.render(**kwargs)

    def open_tpl_files(self):
        self.files = self._get_tpl_dir_files()

        for k, v in self.files.items():
            try:
                with open(v) as f:
                    self.tpl[k] = Template(f.read())
            except Exception as e:
                self.logger.error(f"Error opening template: {v} not found")



from setuptools import setup, find_packages

with open("fcp/version.py") as f:
    version = f.read()

_, version = version.split("=")
version = version.strip()

version = version[1:-1]

with open("README.md") as f:
    long_description = f.read()

setup(
    name="fcp",
    description="CAN bus manager",
    version=version,
    author="Joao Freitas",
    author_email="joaj.freitas@gmail.com",
    license="GPLv3",
    url="https://gitlab.com/joajfreitas/fcp-core",
    download_url="https://gitlab.com/joajfreitas/fcp-core/-/archive/v0.32/fcp-core-v0.29.tar.gz",
    packages=find_packages(),
    entry_points={"console_scripts": ["fcp = fcp.__main__:main",],},
    install_requires=["click", "PySide2", "jinja2", "cantools"],
    long_description=long_description,
    long_description_content_type="text/markdown"
)



from typing import *
import json
import struct
import datetime

class CANMessage:
    def __init__(self, sid: int, dlc: int,  timestamp: int, 
            data16:  List[int] = [],
            data64 = -1):
        
        assert isinstance(sid, int) == True
        self.sid = sid
        self.dlc = dlc

        if data64 == -1:
            assert len(data16) == 4
            self.data64 = sum([data << (16*i) for i,data in enumerate(data16)])
            #assert self.data64 != 0
        else:
            self.data64 = data64

        self.timestamp = timestamp

    def get_data8(self):
        data = []
        for i in range(8):
            data.append((self.data64 >> (8*i)) & 0xFF)

        return data
    
    def get_data16(self):
        return [(self.data64 >> 16*i) & 0xFFFF for i in range(4)]

    def get_data64(self):
        return self.data64

    def encode_json(self):
        """
            Encode message as a utf-8 byte string encoded JSON object. 
            Example: {'sid': 100, 'dlc': 8, 'data': [1,2,3,4], 'timestamp': 1}
            
        """
        return bytes(
            json.dumps(
                {
                    "sid": self.sid,
                    "dlc": self.dlc,
                    "data": self.get_data16(),
                    "timestamp": self.timestamp,
                }
            ),
            "utf-8",
        )

    def encode_bstring(self):
        """
        Encode message as a utf-8 byte string encoded comma separated list of fields.
        Example: 100,8,1,2,3,4,1.
        Encodes sid: 100, dlc: 8, data[0]: 1, data[1]: 2, data[2]: 3, data[3]: 4, timestamp: 1

        """
        data = self.get_data16()
        return bytes(
            str(self.sid)+","+str(self.dlc)+","+str(data[0])+","+str(data[1])+","+str(data[2])+","+str(data[3])+","+str(self.timestamp)+"\n",
            "utf-8",
        )
    
    def encode_struct(self):
        """
        Encode message as a CAN struct:
        
        .. code:: C

            typedef struct {
                union {
                    struct {
                            uint16_t dev_id:5; // Least significant
                            // First bit of msg_id determines if msg is reserved or not.
                            // 0 == reserved (higher priority) (0-31 decimal)
                            uint16_t msg_id:6; // Most significant
                    };
                    uint16_t sid;
                };
                uint16_t dlc:4;
                uint16_t data[4];
            } CANdata;

        """

        s = struct.pack("HH4HI", self.sid, self.dlc, *self.get_data16(), self.timestamp)
        return s

    @staticmethod
    def decode_json(j):
        """
        Decode JSON encoded message into CANMessage.

        See encode_json for details on the JSON format used.
        """

        d = json.loads(j.decode())
        msg = CANMessage(d.get("sid"),d.get("dlc"),d.get("timestamp"),data16=d.get("data"))

        if (msg.sid and msg.dlc and msg.data64 and msg.timestamp) == None:
            raise ValueError("Received invalid message: " + j.decode())
            return

        return msg

    @staticmethod
    def decode_bstring(bstring):
        """
        Decode bytestring encoded message into CANMessage.

        See encode_bstring for details on the string format used.
        """

        sid, dlc, *data, timestamp = bstring.decode().split(",")
        data = [int(d) for d in data]
        sid = int(sid)
        dlc = int(dlc)
        timestamp = int(timestamp)
        msg = CANMessage(sid,dlc,timestamp, data16=data)
        return msg

    @staticmethod
    def decode_CANmessage(message):
        msg = CANMessage()
        msg.sid = message.sid
        msg.dlc = message.dlc
        msg.data = [0, 0, 0, 0]
        for i in range(4):
            msg.data[i] = (message.data >> i * 16) & 0xFFFF
        return msg

    @staticmethod
    def decode_struct(s):
        """
        Decode C struct encoded message into CANMessage.

        See encode_struct for details on the struct format used.
        """
        sid, dlc, *data, timestamp = struct.unpack("HH4HI", s)

        msg = CANMessage(sid, dlc, timestamp, data16=data)

        return msg

    @staticmethod
    def decode_kvaser_csv(csv, start_time):
        try:
            line = csv.split(",")
            timestamp = start_time + datetime.timedelta(seconds=float(line[0]))
            sid = int(line[2])
            dlc = int(line[4])
            data8 = line[5:13]
            data64 = sum([(0 if d == '' else int(d)) << (i*8) for i, d in enumerate(data8)])
            data = []
            for i in range(4):
                data.append((data64 >> 16*i) & 0xFFFF)

            return CANMessage(sid, dlc, timestamp, data16=data)

        except Exception as e:
            return None

    def __repr__(self):
        return str(self.encode_json())[1:]


def test_encode_json():
    msg = CANMessage(100, 8, 100, data16=[1,2,3,4])
    j = json.loads(msg.encode_json())

    assert j["sid"] == 100
    assert j["dlc"] == 8
    assert j["data"] == [1,2,3,4]
    assert j["timestamp"] == 100

def test_encode_bstring():
    msg = CANMessage(100, 8, 100, data16=[1,2,3,4])
    js = msg.encode_bstring().split(b",")
    js = [int(j) for j in js]

    assert js[0] == 100
    assert js[1] == 8
    assert js[2] == 1
    assert js[3] == 2 
    assert js[4] == 3 
    assert js[5] == 4 
    assert js[6] == 100

def test_encode_struct():
    msg = CANMessage(100, 8, 100, data16=[1,2,3,4])
    msg = CANMessage.decode_struct(msg.encode_struct())

    assert msg.sid == 100
    assert msg.dlc == 8
    assert msg.get_data16() == [1,2,3,4]
    assert msg.timestamp == 100

def test_decode_json():
    msg = CANMessage(100, 8, 100, data16=[1,2,3,4])
    msg = CANMessage.decode_json(msg.encode_json())

    assert msg.sid == 100
    assert msg.dlc == 8
    assert msg.get_data16() == [1,2,3,4]
    assert msg.timestamp == 100

def test_decode_bstring():
    msg = CANMessage(100, 8, 100, data16=[1,2,3,4])
    msg = CANMessage.decode_bstring(msg.encode_bstring())

    assert msg.sid == 100
    assert msg.dlc == 8
    assert msg.get_data16() == [1,2,3,4]
    assert msg.timestamp == 100

def test_decode_kvaser_csv():
    msg = CANMessage.decode_kvaser_csv("0.31947,1,1099,2,8,20,178,20,178,61,74,0,0,1", datetime.datetime(year=2019, month=9, day=3))

    assert msg.sid == 1099
